@@ -1,4 +1,4 @@
-// $Id: main_thread.c,v 1.20 2006/08/23 20:27:52 khlut Exp $
+// $Id: main_thread.c,v 1.21 2006/09/19 09:09:51 khlut Exp $
 // Copyright 2005 Elphel, Inc.
 //
 // This file is part of GenReS.
@@ -544,13 +544,20 @@ int32   NPP_Write(NPP instance, NPStream *stream, int32 offset, int32 len, void
 {
     Debug("%p NPP_Write\n", instance->pdata);
     int i= write(Pd->stream,buffer,len);
+    if(i<=0) i=errno==EAGAIN?0:-1;
     Debug("written %d\n",i);
-    if(i<=0) return errno==EAGAIN?0:-1;
     return i;
 }
 void    NPP_Print(NPP instance, NPPrint* printInfo)
 {
-    Debug("%p NPP_Print\n", instance->pdata);
+    Debug("%p NPP_Print mode=%d\n", instance->pdata, printInfo->mode);
+    if(printInfo->mode == NP_EMBED) {
+	FILE *fp=((NPPrintCallbackStruct*)printInfo->print.embedPrint.platformPrint)->fp;
+	if(Pd->slavectrl)fprintf(Pd->slavectrl,"PRINTEMBED\n");
+    }else if(printInfo->mode == NP_FULL) {
+	if(Pd->slavectrl)fprintf(Pd->slavectrl,"PRINTFULL\n");
+	printInfo->print.fullPrint.pluginPrinted=TRUE;
+    }
 }
 void    NPP_URLNotify(NPP instance, const char* url, NPReason reason, void* notifyData)
 {
@@ -608,6 +615,16 @@ void * PR_CALLBACK HandleEvent(ObserveEvent* event)
     Debug("%p HandleEvent \"%s\"\t", event->pd, event->str+2);
 
     char *name= event->str+2;
+    NPObject *obj;
+    NPVariant val;
+    if(*name == '{') {
+	STRINGZ_TO_NPVARIANT( name, val );
+	NPN.getvalue(event->pd->instance, NPNVPluginElementNPObject, &obj);
+	Debug("Eval\n");
+	NPN.evaluate(event->pd->instance, obj, &(val.value.stringValue), 0);
+	NPN.releaseobject(obj);
+	return 0;
+    }
     char *value= pair(&name, " \t", "=:\n", "\n");
     if(!value || !*name) {
 	if(strcmp(event->str+2,"CLOSE")==0) {
@@ -628,9 +645,7 @@ void * PR_CALLBACK HandleEvent(ObserveEvent* event)
 	return 0;
     }
 
-    NPVariant val;
     STRINGZ_TO_NPVARIANT( value, val );
-    NPObject *obj;
     if(event->pd->instance && NPN.getvalue(event->pd->instance, NPNVPluginElementNPObject, &obj)==NPERR_NO_ERROR) {
 	if( storage_set( &(event->pd->vars), NPN.getstringidentifier(name), &val ) ) {
 	    //if(strcmp(name,"debug")==0) debug=*value-'0';
